"""
LsSysFirmware - command ``ls /sys/firmware``
============================================

The ``ls -lanR /sys/firmware`` command provides information for the listing of
the ``/sys/firmware`` directory.

See :class:`insights.parsers.ls.FileListing` for more information.

"""

from insights import CommandParser, FileListing, parser
from insights.specs import Specs


@parser(Specs.ls_sys_firmware)
class LsSysFirmware(CommandParser, FileListing):
    """
    Parses output of ``ls -lanR /sys/firmware`` command.

    .. warning::

        For Insights Advisor Rules, it's recommended to use the
        :class:`insights.parsers.ls.LSlanR` and add the ``"/sys/firmware"`` to
        the filter list of `Specs.ls_lanR_dirs` instead.

    Sample directory listing::

        /sys/firmware:
        total 0
        drwxr-xr-x.  5 0 0 0 Dec 22 17:56 .
        dr-xr-xr-x. 13 0 0 0 Dec 22 17:56 ..
        drwxr-xr-x.  5 0 0 0 Dec 22 17:56 acpi
        drwxr-xr-x.  3 0 0 0 Dec 22 17:57 dmi
        drwxr-xr-x. 10 0 0 0 Dec 22 17:57 memmap

        /sys/firmware/acpi:
        total 0
        drwxr-xr-x. 5 0 0    0 Dec 22 17:56 .
        drwxr-xr-x. 5 0 0    0 Dec 22 17:56 ..
        drwxr-xr-x. 6 0 0    0 Feb 10 15:54 hotplug
        drwxr-xr-x. 2 0 0    0 Feb 10 15:54 interrupts
        -r--r--r--. 1 0 0 4096 Feb 10 15:54 pm_profile
        drwxr-xr-x. 3 0 0    0 Dec 22 17:56 tables

    Examples:
        >>> type(ls_sys_firmware)
        <class 'insights.parsers.ls_sys_firmware.LsSysFirmware'>
        >>> "acpi" in ls_sys_firmware
        False
        >>> "/sys/firmware/acpi" in ls_sys_firmware
        True
        >>> ls_sys_firmware.dirs_of("/sys/firmware")
        ['.', '..', 'acpi', 'dmi', 'memmap']
        >>> ls_sys_firmware.files_of("/sys/firmware/acpi")
        ['pm_profile']
    """
    __root_path = '/sys/firmware'



"""
Clean the collected specs (files/commands/datasources)
======================================================

The following processes will be applied to clean the collected specs:

    - Redaction
      This is a must-be-done operation to all the collected specs.

    - Obfuscation
      Obfuscate the IP or Hostname appears in the spec content according to the
      specs native requirement and user configuration.

"""
import logging
import hashlib
import json
import os
import re
import six
import socket
import struct

# TODO: getting RHSM facts file from the InsightsConstants directly
# from insights.client.constants import InsightsConstants as constants
from insights.util.hostname import determine_hostname
from insights.util.posix_regex import replace_posix

logger = logging.getLogger(__name__)

DEFAULT_PASSWORD_REGEXS = [
    r"(password[a-zA-Z0-9_]*)(\s*\:\s*\"*\s*|\s*\"*\s*=\s*\"\s*|\s*=+\s*|\s*--md5+\s*|\s*)([a-zA-Z0-9_!@#$%^&*()+=/-]*)",
    r"(password[a-zA-Z0-9_]*)(\s*\*+\s+)(.+)",
]
"""The regex for password removal, which is read from the "/etc/insights-client/.exp.sed"."""


def write_report(report, report_file, mode=0o644):
    # Get the current umask
    umask = os.umask(0o022)
    # Reset the umask
    os.umask(umask)
    try:
        with open(report_file, 'w') as fp:
            if isinstance(report, dict):
                json.dump(report, fp)
            elif isinstance(report, list):
                for line in report:
                    fp.write("{0}\n".format(line))
        # Change the file mode per the current umask
        os.chmod(report_file, mode & ~umask)
    except (IOError, OSError) as e:
        logger.error('Could not write to %s: %s', report_file, str(e))


class Cleaner(object):
    def __init__(self, config, rm_conf, fqdn=None):
        self.report_dir = '/tmp'
        # Obfuscation - set: ip and hostname only
        self.obfuscate = set()
        self.obfuscate.add('ip') if config and config.obfuscate else None
        self.obfuscate.add('hostname') if config and config.obfuscate_hostname else None

        # File Content Redaction
        # - Pattern redaction
        rm_conf = rm_conf or {}
        exclude = rm_conf.get('patterns', [])
        regex = False
        if isinstance(exclude, dict) and exclude.get('regex'):
            exclude = [r'%s' % replace_posix(i) for i in exclude['regex']]
            regex = True
        self.redact = dict(exclude=exclude, regex=regex)

        # - Keyword replacement redact information
        #   Keyword replacement does NOT depend on "obfuscate=True"
        keywords = rm_conf.get('keywords')
        self.kw_db = dict()  # keyword database
        self.kws = set()     # keywords that have been replaced
        self._keywords2db(keywords)

        # Obfuscation
        fqdn = fqdn if fqdn else determine_hostname()
        name_list = fqdn.split('.')
        self.hostname = name_list[0]
        self.fqdn = fqdn
        self.domain = None if len(name_list) <= 1 else '.'.join(name_list[1:])

        # - IP obfuscate information
        self.ip_db = dict()  # IP database
        self.start_ip = '10.230.230.1'

        # - Hostname obfuscate information
        self.hn_db = dict()  # hostname database
        self.hostname_count = 0
        self.obfuscated_fqdn = None
        self.obfuscated_domain = 'example.com'  # right now this needs to be a 2nd level domain, like foo.com, example.com, domain.org, etc.

        # - Domain name obfuscate information
        self.dn_db = dict()  # domain name database
        self.domain_count = 0

        if config and config.obfuscate_hostname and self.fqdn:
            self._domains2db()
            hashed_hostname = hashlib.sha1(
                    self.fqdn.encode('utf-8')
                    if six.PY3 else self.fqdn).hexdigest()[:12]
            self.obfuscated_fqdn = '{0}.example.com'.format(hashed_hostname)
            self.hostname_count += 1
            self.hn_db[self.obfuscated_fqdn] = self.fqdn
            # per https://access.redhat.com/documentation/en-us/red_hat_insights/2023/html/client_configuration_guide_for_red_hat_insights/con-insights-client-cg-data-obfuscation_insights-cg-obfuscation#proc-obfuscating-hostname_insights-cg-obfuscation
            # only `hostname` is obfuscated

    ###########################
    #   IP functions          #
    ###########################

    def _ip2int(self, ipstr):
        # converts a dotted decimal IP address into an integer that can be incremented
        return struct.unpack('!I', socket.inet_aton(ipstr))[0]

    def _int2ip(self, num):
        # converts an integer stored in the IP database into a dotted decimal IP
        return socket.inet_ntoa(struct.pack('!I', num))

    def _ip2db(self, ip):
        '''
        adds an IP address to the IP database and returns the obfuscated entry, or returns the
        existing obfuscated IP entry
        FORMAT:
        {$obfuscated_ip: $original_ip,}
        '''
        ip_num = self._ip2int(ip)
        ip_found = False
        db = self.ip_db
        for k, v in db.items():
            if v == ip_num:
                ret_ip = self._int2ip(k)
                ip_found = True
        if ip_found:                # the entry already existed
            return ret_ip
        else:                       # the entry did not already exist
            if len(self.ip_db) > 0:
                new_ip = max(db.keys()) + 1
            else:
                new_ip = self._ip2int(self.start_ip)
            db[new_ip] = ip_num

            return self._int2ip(new_ip)

    def _sub_ip(self, line):
        '''
        This will substitute an obfuscated IP for each instance of a given IP in a file
        This is called in the self._clean_line function, along with user _sub_* functions to scrub a given
        line in a file.
        It scans a given line and if an IP exists, it obfuscates the IP using _ip2db and returns the altered line
        '''
        try:
            pattern = r"(((\b25[0-5]|\b2[0-4][0-9]|\b1[0-9][0-9]|\b[1-9][0-9]|\b[1-9]))(\.(\b25[0-5]|\b2[0-4][0-9]|\b1[0-9][0-9]|\b[1-9][0-9]|\b[0-9])){3})"
            ips = [each[0] for each in re.findall(pattern, line)]
            if len(ips) > 0:
                for ip in sorted(ips, key=len, reverse=True):
                    # skip loopback (https://github.com/RedHatInsights/insights-core/issues/3230#issuecomment-924859845)
                    if ip != "127.0.0.1" and ip in line:
                        new_ip = self._ip2db(ip)
                        logger.debug("Obfuscating IP - %s > %s", ip, new_ip)
                        line = line.replace(ip, new_ip)
            return line
        except Exception as e:  # pragma: no cover
            logger.warning(e)
            raise Exception('SubIPError: Unable to Substitute IP Address - %s', ips)

    def _sub_ip_netstat(self, line):
        '''
        Special version of _sub_ip for netstat to preserve spacing
        '''
        try:
            pattern = r"(((\b25[0-5]|\b2[0-4][0-9]|\b1[0-9][0-9]|\b[1-9][0-9]|\b[1-9]))(\.(\b25[0-5]|\b2[0-4][0-9]|\b1[0-9][0-9]|\b[1-9][0-9]|\b[0-9])){3})"
            ips = [each[0] for each in re.findall(pattern, line)]
            if len(ips) > 0:
                for ip in sorted(ips, key=len, reverse=True):
                    # skip loopback (https://github.com/RedHatInsights/insights-core/issues/3230#issuecomment-924859845)
                    if ip != "127.0.0.1" and ip in line:
                        ip_len = len(ip)
                        new_ip = self._ip2db(ip)
                        new_ip_len = len(new_ip)
                        logger.debug("Obfuscating IP - %s > %s", ip, new_ip)
                        # pad or remove spaces to allow for the new length
                        if ip_len > new_ip_len:
                            numspaces = ip_len - new_ip_len
                            line = line.replace(ip, new_ip)

                            # shift past port specification to add spaces
                            idx = line.index(new_ip) + new_ip_len
                            c = line[idx]
                            while c != " ":
                                idx += 1
                                if idx == len(line):
                                    idx = len(line) - 1
                                    break
                                c = line[idx]
                            line = line[0:idx] + numspaces * " " + line[idx:]

                        elif new_ip_len > ip_len:
                            numspaces = new_ip_len - ip_len
                            line = line.replace(ip, new_ip)

                            # shift past port specification to skip spaces
                            idx = line.index(new_ip) + new_ip_len
                            c = line[idx]
                            while c != " ":
                                idx += 1
                                if idx == len(line):
                                    break
                                c = line[idx]
                            line = line[0:idx] + line[(idx + numspaces):]

                        else:
                            line = line.replace(ip, new_ip)
            return line
        except Exception as e:  # pragma: no cover
            logger.warning(e)
            raise Exception('SubIPError: Unable to Substitute IP Address - %s', ip)

    #############################
    # Hostname Domain Functions #
    #############################

    def _domains2db(self):
        # adds any additional domainnames to the domain database to be searched for
        try:
            # we will add the root domain for an FQDN as well.
            if self.domain is not None:
                self.dn_db[self.obfuscated_domain] = self.domain
                logger.debug("Obfuscated Domain Created - %s -> %s" % (self.domain, self.obfuscated_domain))

            self.domain_count = len(self.dn_db)
            return True
        except Exception as e:  # pragma: no cover
            logger.warning(e)

    def _hn2db(self, hn):
        '''
        This will add a hostname for a hostname for an included domain or return an existing entry
        '''
        db = self.hn_db
        hn_found = False
        for k, v in db.items():
            if v == hn:  # the hostname is in the database
                ret_hn = k
                hn_found = True
        if hn_found:
            return ret_hn
        else:
            self.hostname_count += 1  # we have a new hostname, so we increment the counter to get the host ID number
            o_domain = self.obfuscated_domain
            for od, d in self.dn_db.items():
                if d in hn:  # never false
                    o_domain = od
            new_hn = "host%s.%s" % (self.hostname_count, o_domain)
            self.hn_db[new_hn] = hn
            return new_hn

    def _sub_hostname(self, line):
        '''
        This will replace the exact hostname and all instances of the domain name with the obfuscated alternatives.
        Example:
        '''
        if not line:
            return line
        try:
            for od, d in self.dn_db.items():
                # regex = re.compile(r'\w*\.%s' % d)
                regex = re.compile(r'(?![\W\-\:\ \.])[a-zA-Z0-9\-\_\.]*\.%s' % d)
                hostnames = [each for each in regex.findall(line)]
                if len(hostnames) > 0:
                    for hn in hostnames:
                        new_hn = self._hn2db(hn)
                        logger.debug("Obfuscating FQDN - %s > %s", hn, new_hn)
                        line = line.replace(hn, new_hn)
            if self.hostname:
                line = line.replace(self.hostname, self._hn2db(self.fqdn))  # catch any non-fqdn instances of the system hostname
            return line
        except Exception as e:  # pragma: no cover
            logger.warning(e)
            raise Exception('SubHostnameError: Unable to Substitute Hostname/Domainname')

    ###########################
    #   Keyword functions     #
    ###########################

    def _keywords2db(self, keywords):
        # processes optional keywords to add to be obfuscated
        try:
            if keywords:
                k_count = 0
                for keyword in keywords:
                    o_kw = "keyword%s" % k_count
                    self.kw_db[keyword.rstrip()] = o_kw
                    logger.debug("Added Obfuscated Keyword - %s", o_kw)
                    k_count += 1
                logger.debug("Added Keyword Contents from Customer's configuration")

        except Exception as e:  # pragma: no cover
            logger.warning(e)

    def _sub_keywords(self, line):
        # this will substitute out any keyword entries on a given line
        if not line:
            return line
        for k, v in self.kw_db.items():
            if k in line:
                line = line.replace(k, v)
                self.kws.add(k)
                logger.debug("Replacing Keyword - %s > %s", k, v)
        return line

    ###########################
    #   Main functions        #
    ###########################

    def _obfuscate_line(self, line, obf_funcs):
        # obfuscate line for possible hostname, ip
        if not line:
            return line
        for func in obf_funcs:
            tmp_line = func(line)
            line = tmp_line
        return line

    def _redact_line(self, line):
        # redact line per the file-content-redaction.yaml
        if not line:
            return line
        # 1. patterns removal
        find = re.search if self.redact['regex'] else lambda x, y: x in y
        if any(find(pat, line) for pat in self.redact.get('exclude', [])):
            logger.debug("Pattern matched, removing line: %s" % line.strip())
            # patterns found, remove it
            return None
        # 2. password removal
        # TODO - refine the workaround
        if not line.strip().endswith(".withoutpassword"):  # workaround special lines
            for regex in DEFAULT_PASSWORD_REGEXS:
                tmp_line = line
                line = re.sub(regex, r"\1\2********", tmp_line)
                if line != tmp_line:
                    break
        # 3. keyword replacement redaction
        return self._sub_keywords(line)

    def get_obfuscate_functions(self, filename='', no_obfuscate=None):
        """
        Return the list of required obfuscation function according to the
        filename and configuration.  By default, it returns:
            - [] when obfuscate=False
            - [self._sub_ip] when obfuscate=True Only
            - [self._sub_hostname, self._sub_ip] when obfuscate_hostname=True
        """
        obf_funcs = []
        # Get the actual obfuscate list setting for this file
        obfs = set(self.obfuscate) - set(no_obfuscate or [])
        # IP obfuscation entry
        obf_funcs.append(self._sub_ip_netstat if filename.endswith("netstat_-neopa") else self._sub_ip) if "ip" in obfs else None
        # Hostname obfuscation entry
        obf_funcs.append(self._sub_hostname) if "hostname" in obfs else None
        return obf_funcs

    def clean_content(self, lines, obf_funcs=None, no_redact=False):
        """
        Clean lines one by one according to the configuration, the cleaned
        lines will be returned.
        """
        def _clean_line(_line):
            # 1. Do Redaction by default, unless "no_redact=True"
            if _line and not no_redact:
                _line = self._redact_line(_line)
            # 2. Do Obfuscation as per the "obf_funcs"
            _line = self._obfuscate_line(_line, obf_funcs or [])
            return _line

        # handle single string
        if not isinstance(lines, list):
            return _clean_line(lines)

        result = []
        for line in lines:
            line = _clean_line(line)
            result.append(line) if line is not None else None
        if result and list(filter(None, result)):
            # When there are some lines Truth
            return result
        # All lines blank
        return []

    def clean_file(self, _file, no_obfuscate=None, no_redact=False):
        """
        Clean a file according to the configuration, the file will be updated
        directly with the cleaned content.
        """
        logger.debug('Cleaning %s ...' % _file)

        if os.path.exists(_file) and not os.path.islink(_file):
            # Process the file
            raw_data = content = None
            obf_funcs = self.get_obfuscate_functions(_file, no_obfuscate)
            try:
                with open(_file, 'r') as fh:
                    raw_data = fh.readlines()
                    content = self.clean_content(raw_data, obf_funcs, no_redact)
            except Exception as e:  # pragma: no cover
                logger.warning(e)
                raise Exception("Error: Cannot Open File for Cleaning: %s" % _file)
            # Store it
            try:
                if raw_data:
                    if content:
                        with open(_file, 'wb') as fh:
                            for line in content:
                                fh.write(line.encode('utf-8') if six.PY3 else line)
                    else:
                        # Remove Empty file
                        logger.debug('Removing %s, as it\'s empty after cleaning' % _file)
                        os.remove(_file)
            except Exception as e:  # pragma: no cover
                logger.warning(e)
                raise Exception("Error: Cannot Write to File: %s" % _file)

    def generate_rhsm_facts(self, rhsm_facts_file):
        logger.info('Writing RHSM facts to %s ...', rhsm_facts_file)

        hn_block = []
        for k, v in self.hn_db.items():
            hn_block.append({'original': v, 'obfuscated': k})

        kw_block = []
        for k in self.kws:
            kw_block.append({'original': k, 'obfuscated': self.kw_db[k]})

        ip_block = []
        for k, v in self.ip_db.items():
            ip_block.append(
                {
                    'original': self._int2ip(v),
                    'obfuscated': self._int2ip(k)
                })

        facts = {
            'insights_client.hostname': self.fqdn,
            'insights_client.obfuscate_ip_enabled': 'ip' in self.obfuscate,
            'insights_client.ips': json.dumps(ip_block),
            'insights_client.obfuscate_hostname_enabled': 'hostname' in self.obfuscate,
            'insights_client.hostnames': json.dumps(hn_block),
            'insights_client.keywords': json.dumps(kw_block),
        }

        write_report(facts, rhsm_facts_file)

    def generate_ip_report(self, archive_name):
        try:
            ip_report_file = os.path.join(self.report_dir, "%s-ip.csv" % archive_name)
            logger.info('Creating IP Report - %s', ip_report_file)
            lines = ['Obfuscated IP,Original IP']
            for k, v in self.ip_db.items():
                lines.append('{0},{1}'.format(self._int2ip(k), self._int2ip(v)))
        except Exception as e:  # pragma: no cover
            logger.exception(e)
            raise Exception('CreateReport Error: Error Creating IP Report')

        write_report(lines, ip_report_file)

        logger.info('Completed IP Report.')

    def generate_hn_report(self, archive_name):
        try:
            hn_report_file = os.path.join(self.report_dir, "%s-hostname.csv" % archive_name)
            logger.info('Creating Hostname Report - %s', hn_report_file)
            lines = ['Obfuscated Hostname,Original Hostname']
            if self.hostname_count > 0:
                for k, v in self.hn_db.items():
                    lines.append('{0},{1}'.format(k, v))
            else:
                lines.append('None,None')
        except Exception as e:  # pragma: no cover
            logger.exception(e)
            raise Exception('CreateReport Error: Error Creating Hostname Report')

        write_report(lines, hn_report_file)

        logger.info('Completed Hostname Report.')

    def generate_kw_report(self, archive_name):
        try:
            kw_report_file = os.path.join(self.report_dir, "%s-keyword.csv" % archive_name)
            logger.info('Creating Keyword Report - %s', kw_report_file)
            lines = ['Replaced Keyword,Original Keyword']
            for k in self.kws:
                lines.append('{0},{1}'.format(k, self.kw_db[k]))
        except Exception as e:  # pragma: no cover
            logger.exception(e)
            raise Exception('CreateReport Error: Error Creating Keyword Report')

        write_report(lines, kw_report_file)

        logger.info('Completed Keyword Report.')

    def generate_report(self, archive_name, rhsm_facts_file):
        # Always generate the rhsm.facts files
        self.generate_rhsm_facts(rhsm_facts_file)
        if 'ip' in self.obfuscate:
            self.generate_ip_report(archive_name)
        if 'hostname' in self.obfuscate:
            self.generate_hn_report(archive_name)
        if self.kws:
            self.generate_kw_report(archive_name)



"""
Mlx4Port Combiner for the Mlx4Port Parser
=========================================

Combiner for the :class:`insights.parsers.mlx4_port.Mlx4Port` parser.

This parser is multioutput, one parser instance for each port
file.  This combiner puts all of them back together and presents
them as a dict where the keys are the port names, and the contents
of the port name file are the lines in each file stored as a list.

This class inherits all methods and attributes from the ``dict`` object.

Examples:
    >>> type(mlx4port)
    <class 'insights.combiners.mlx4_port.Mlx4Port'>
    >>> mlx4port['mlx4_port1']
    ['ib']
    >>> sorted(mlx4port.keys())
    ['mlx4_port1', 'mlx4_port2']

"""

from .. import combiner
from insights.parsers.mlx4_port import Mlx4Port as Mlx4PortParser


@combiner(Mlx4PortParser)
class Mlx4Port(dict):
    """
    Combiner for the mlx4_port parser.
    """
    def __init__(self, mlx4_port):
        super(Mlx4Port, self).__init__()
        for port in mlx4_port:
            self.update({port.name: port.contents})



"""
Ls Etc Directory
================

Parsers provided in this module includes:

LsEtc - command ``ls -lan /etc </etc/sub-dirs>``
------------------------------------------------

LsEtcSsh - command ``/bin/ls -lanL /etc/ssh``
---------------------------------------------
"""
from .. import parser, CommandParser
from .. import FileListing
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.ls_etc)
class LsEtc(CommandParser, FileListing):
    """
    .. warning::
        This class is deprecated and will be removed from 3.5.0.
        Please use the :class:`insights.parsers.ls.LSlan` instead.

    Parses output of ``ls -lan /etc </etc/sub-directories>`` command.

    The ``ls -lan /etc </etc/sub-directories>`` command provides information for
    the listing of the ``/etc`` directory and specified sub-directories.
    See ``FileListing`` class for additional information.

    Sample ``ls -lan /etc/sysconfig /etc/rc.d/rc3.d`` output::

        /etc/sysconfig:
        total 96
        drwxr-xr-x.  7 0 0 4096 Jul  6 23:41 .
        drwxr-xr-x. 77 0 0 8192 Jul 13 03:55 ..
        drwxr-xr-x.  2 0 0   41 Jul  6 23:32 cbq
        drwxr-xr-x.  2 0 0    6 Sep 16  2015 console
        -rw-------.  1 0 0 1390 Mar  4  2014 ebtables-config
        -rw-r--r--.  1 0 0   72 Sep 15  2015 firewalld
        lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 grub -> /etc/default/grub

        /etc/rc.d/rc3.d:
        total 4
        drwxr-xr-x.  2 0 0   58 Jul  6 23:32 .
        drwxr-xr-x. 10 0 0 4096 Sep 16  2015 ..
        lrwxrwxrwx.  1 0 0   20 Jul  6 23:32 K50netconsole -> ../init.d/netconsole
        lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 S10network -> ../init.d/network
        lrwxrwxrwx.  1 0 0   15 Jul  6 23:32 S97rhnsd -> ../init.d/rhnsd

    Examples:

        >>> "sysconfig" in ls_etc
        False
        >>> "/etc/sysconfig" in ls_etc
        True
        >>> len(ls_etc.files_of('/etc/sysconfig'))
        3
        >>> ls_etc.files_of("/etc/sysconfig")
        ['ebtables-config', 'firewalld', 'grub']
        >>> ls_etc.dirs_of("/etc/sysconfig")
        ['.', '..', 'cbq', 'console']
        >>> ls_etc.specials_of("/etc/sysconfig")
        []
        >>> ls_etc.total_of("/etc/sysconfig")
        96
        >>> ls_etc.dir_entry('/etc/sysconfig', 'grub') == {'group': '0', 'name': 'grub', 'links': 1, 'perms': 'rwxrwxrwx.', 'raw_entry': 'lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 grub -> /etc/default/grub', 'owner': '0', 'link': '/etc/default/grub', 'date': 'Jul  6 23:32', 'type': 'l', 'dir': '/etc/sysconfig', 'size': 17}
        True
        >>> ls_etc.files_of('/etc/rc.d/rc3.d')
        ['K50netconsole', 'S10network', 'S97rhnsd']
        >>> sorted(ls_etc.listing_of("/etc/sysconfig").keys()) == sorted(['console', 'grub', '..', 'firewalld', '.', 'cbq', 'ebtables-config'])
        True
        >>> sorted(ls_etc.listing_of("/etc/sysconfig")['console'].keys()) == sorted(['group', 'name', 'links', 'perms', 'raw_entry', 'owner', 'date', 'type', 'dir', 'size'])
        True
        >>> ls_etc.listing_of("/etc/sysconfig")['console']['type']
        'd'
        >>> ls_etc.listing_of("/etc/sysconfig")['console']['perms']
        'rwxr-xr-x.'
        >>> ls_etc.dir_contains("/etc/sysconfig", "console")
        True
        >>> ls_etc.dir_entry("/etc/sysconfig", "console") == {'group': '0', 'name': 'console', 'links': 2, 'perms': 'rwxr-xr-x.', 'raw_entry': 'drwxr-xr-x.  2 0 0    6 Sep 16  2015 console', 'owner': '0', 'date': 'Sep 16  2015', 'type': 'd', 'dir': '/etc/sysconfig', 'size': 6}
        True
        >>> ls_etc.dir_entry("/etc/sysconfig", "grub")['type']
        'l'
        >>> ls_etc.dir_entry("/etc/sysconfig", "grub")['link']
        '/etc/default/grub'
    """
    def __init__(self, *args, **kwargs):
        deprecated(LsEtc, "Please use the :class:`insights.parsers.ls.LSlan` instead.", "3.5.0")
        super(LsEtc, self).__init__(*args, **kwargs)


@parser(Specs.ls_etc_ssh)
class LsEtcSsh(CommandParser, FileListing):
    """
    .. warning::
        This class is deprecated and will be removed from 3.5.0.
        Please use the :class:`insights.parsers.ls.LSlanL` instead.

    Parses output of ``ls -lanL /etc/ssh`` command.

    Sample ``ls -lanL /etc/ssh`` output::

        total 612
        drwxr-xr-x.   3 0   0    245 Aug 11 14:19 .
        drwxr-xr-x. 138 0   0   8192 Jul 29 19:11 ..
        -rw-r--r--.   1 0   0 577388 Mar 27  2020 moduli
        -rw-r--r--.   1 0   0   1770 Mar 27  2020 ssh_config
        drwxr-xr-x.   2 0   0     28 May 12 17:10 ssh_config.d
        -rw-r-----.   1 0 994    480 May 13 09:58 ssh_host_ecdsa_key
        -rw-r--r--.   1 0   0    162 May 13 09:58 ssh_host_ecdsa_key.pub
        -rw-r-----.   1 0 994    387 May 13 09:58 ssh_host_ed25519_key
        -rw-r--r--.   1 0   0     82 May 13 09:58 ssh_host_ed25519_key.pub
        -rw-r-----.   1 0 994   2578 May 13 09:58 ssh_host_rsa_key
        -rw-r--r--.   1 0   0    554 May 13 09:58 ssh_host_rsa_key.pub
        -rw-------.   1 0   0   4260 Aug 11 14:19 sshd_config

    Examples:
        >>> type(ls_etc_ssh)
        <class 'insights.parsers.ls_etc.LsEtcSsh'>
        >>> ls_etc_ssh.files_of("/etc/ssh")
        ['moduli', 'ssh_config', 'ssh_host_ecdsa_key', 'ssh_host_ecdsa_key.pub', 'ssh_host_ed25519_key', 'ssh_host_ed25519_key.pub', 'ssh_host_rsa_key', 'ssh_host_rsa_key.pub', 'sshd_config']
        >>> ls_etc_ssh.dir_entry('/etc/ssh', 'ssh_host_rsa_key')['perms']
        'rw-r-----.'
    """
    def __init__(self, *args, **kwargs):
        deprecated(LsEtcSsh, "Please use the :class:`insights.parsers.ls.LSlanL` instead.", "3.5.0")
        super(LsEtcSsh, self).__init__(*args, **kwargs)



import pytest

from insights.core import filters
from insights.core.exceptions import SkipComponent
from insights.specs import Specs
from insights.specs.datasources.dir_list import du_dir_list


def setup_function(func):
    if Specs.du_dirs in filters._CACHE:
        del filters._CACHE[Specs.du_dirs]
    if Specs.du_dirs in filters.FILTERS:
        del filters.FILTERS[Specs.du_dirs]

    if func is test_du_dirs_list:
        filters.add_filter(Specs.du_dirs, ["/var/lib/pulp", "/etc/httpd"])
    if func is test_du_dirs_list_no_filter:
        filters.add_filter(Specs.du_dirs, [])


def test_du_dirs_list():
    broker = {}
    result = du_dir_list(broker)
    assert len(result) == 2
    assert '/var/lib/pulp' in result
    assert '/etc/httpd' in result


def test_du_dirs_list_no_filter():
    broker = {}
    with pytest.raises(SkipComponent):
        du_dir_list(broker)



import pytest

from insights.core.plugins import component
from insights.core.exceptions import SkipComponent
from insights.tests import InputData
from insights.tests import run_input_data


@component()
def some_component():
    raise SkipComponent


@pytest.mark.parametrize(
    "store_skips, exceptions_count",
    ((False, 0), (True, 1)),
)
def test_store_skips_false(store_skips, exceptions_count):
    broker = run_input_data(some_component, InputData(), store_skips=store_skips)
    assert len(broker.exceptions) == exceptions_count



import doctest
import pytest

from insights.core.exceptions import ParseException
from insights.parsers import bdi_read_ahead_kb
from insights.tests import context_wrap

BDI_READ_AHEAD_KB = """
128
""".strip()

BDI_READ_AHEAD_KB_INVALID = """
invalid
""".strip()


def test_bdi_read_ahead_kb():
    read_ahead_kb = bdi_read_ahead_kb.BDIReadAheadKB(context_wrap(BDI_READ_AHEAD_KB))
    assert read_ahead_kb.read_ahead_kb == 128


def test_invalid_bdi_read_ahead_kb():
    with pytest.raises(ParseException) as e:
        bdi_read_ahead_kb.BDIReadAheadKB(context_wrap(BDI_READ_AHEAD_KB_INVALID))
    assert "Error: " in str(e)


def test_bdi_read_ahead_kb_doc_examples():
    env = {
        'bdi_read_ahead_kb': bdi_read_ahead_kb.BDIReadAheadKB(context_wrap(BDI_READ_AHEAD_KB)),
    }
    failed, total = doctest.testmod(bdi_read_ahead_kb, globs=env)
    assert failed == 0



"""
NeutronConf - file ``/etc/neutron/neutron.conf``
================================================
"""
from insights.core import IniConfigFile
from insights.core.filters import add_filter
from insights.core.plugins import parser
from insights.specs import Specs

add_filter(Specs.neutron_conf, ["["])

ADDITIONAL_FILTERS = [
    "service_plugins",
    "allow_automatic_dhcp_failover",
    "rpc_workers",
    "api_workers",
    "ipam_driver",
    "agent_down_time",
    "agent_report_interval",
    "router_distributed"
]
add_filter(Specs.neutron_conf, ADDITIONAL_FILTERS)


@parser(Specs.neutron_conf)
class NeutronConf(IniConfigFile):
    """
    This class provides parsing for the file ``/etc/neutron/neutron.conf``.

    Sample input data is in the format::

        [DEFAULT]
        debug = False
        verbose = False
        core_plugin = neutron.plugins.openvswitch.ovs_neutron_plugin.OVSNeutronPluginV2

        [quotas]
        default_quota = -1
        quota_network = 10

        [agent]
        report_interval = 60

        [keystone_authtoken]
        auth_host = ost-controller-lb-del.example.com
        auth_port = 35357

        [database]
        connection = mysql://neutron:dSNneutron01@ost-mysql.example.com/neutron?ssl_ca=/etc/pki/CA/certs/ca.crt

        [service_providers]
        service_provider = LOADBALANCER:Haproxy:neutron.services.loadbalancer.drivers.haproxy.plugin_driver.HaproxyOnHostPluginDriver:default

    Examples:
        >>> type(conf)
        <class 'insights.parsers.neutron_conf.NeutronConf'>
        >>> conf.sections()
        ['quotas', 'agent', 'keystone_authtoken', 'database', 'service_providers']
        >>> conf.has_option('DEFAULT', 'debug')
        True
        >>> conf.get("DEFAULT", "verbose")
        'False'
        >>> conf.get("keystone_authtoken", "auth_host")
        'ost-controller-lb-del.example.com'
        >>> conf.getboolean("DEFAULT", "debug")
        False
        >>> conf.getint("agent", "report_interval")
        60
    """
    pass



import os
import sys
from insights.client.constants import InsightsConstants as constants
from insights.client.apps.ansible.playbook_verifier import verify, load_playbook_yaml, PlaybookVerificationError


def read_playbook():
    """Read in the playbook from stdin."""
    unverified_playbook = ""
    for line in sys.stdin:
        unverified_playbook += line
    return unverified_playbook


try:
    raw_playbook = read_playbook()
    if os.environ.get("SKIP_VERIFY"):
        print(raw_playbook)
        exit(0)

    playbook = load_playbook_yaml(raw_playbook)[0]  # type: dict
    verified_playbook = verify(playbook)  # type: dict
except PlaybookVerificationError as err:
    sys.stderr.write(err.message)
    sys.exit(constants.sig_kill_bad)

print(raw_playbook)



"""
KubepodsCpuQuota - CPU quota for each Kubernetes pod
====================================================

This parser reads the content of ``/sys/fs/cgroup/cpu/kubepods.slice/kubepods-burstable.slice/*.slice/cpu.cfs_quota_us``.
"""
from insights.core import Parser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.kubepods_cpu_quota)
class KubepodsCpuQuota(Parser):
    """
    Class ``KubepodsCpuQuota`` parses the content of the ``/sys/fs/cgroup/cpu/kubepods.slice/kubepods-burstable.slice/kubepods-burstable-pod*.slice/cpu.cfs_quota_us``.

    Attributes:
        cpu_quota (int): It is used to show the value of cpu quota for a particular pod in a Kubernetes cluster or an OpenShift cluster.

    A typical sample of the content of this file looks like::

        -1

    Examples:
        >>> type(kubepods_cpu_quota)
        <class 'insights.parsers.kubepods_cpu_quota.KubepodsCpuQuota'>
        >>> kubepods_cpu_quota.cpu_quota
        -1
    """

    def parse_content(self, content):
        if len(content) != 1 or not content[0].strip('-').isdigit():
            raise ParseException("Error: ", content if content else 'empty file')
        self.cpu_quota = int(content[0].strip())
